#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "stack.h"

#define MAX_EQU_LEN 100

/*
 * Returns a number corresponding the the preccedence 
 * of different arithmetic operators. * has a higher 
 * precedence that / which is greater than %, +, -
 * in that order. Higher precedence should return a higher
 * number so say 5 for * 1 for - and 0 for a non operator
 */
static char prec(char operator) {
	if(operator == '*') {
		return 5;
	} else if(operator == '/') {
		return 4;
	} else if(operator == '%') {
		return 3;
	} else if(operator == '+') {
		return 2;
	} else if(operator == '-') {
		return 1;
	} else {
		return 0; // Non-operator
	}
}

/*
 * Returns 1 if the string passed is an inteter
 * 0 otherwise
 */
static int isNumeric(char *num) {
	if(isdigit(num) == 0) {
		return 0;
	} else {
		return 1;
	}
}

/*
 * converts a valid infix expression into postfix
 * Hint put a space between numbers and operators 
 * so you can use strtok in evaluate_postfix
 * ex: get 7 8 + vs 78+
 *
 * This is done with the following algorithm
 *
 * for each token in the string
 *     if the next token is a number
 *         append it to the postfix string
 *     else if the next token is a left paren
 *          push it onto the stack
 *     else if the next token is a right paren
 *          while the stack is not empty and the left paren is not at the top of the stack
 *              pop the next operator off of the stack
 *              append it to the postfix string
 *          pop the left paren off of the stack and discard it
 *     else if the next token is an operator (+, -, *, /)
 *         while the stack is not empty and the operator at the top of the stack has a higher precedence than the token
 *              pop the top element off of the stack and append it to the postfix string
 *         push the current token onto the stack
 *
 * while there are elements remaining on the stack
 *     pop the top element off of the stack and append it to the postfix string
 */

char* infix_to_postfix(char* infix) {
	stack* stack = create_stack();
	char* postfix = malloc(sizeof(char) * MAX_EQU_LEN);

	int i;
	int pos = 0;
	for(i=0; infix[i] != '\n'; i++) {
		printf("\n::::::::::::::::::HELLO BOYS AND GIRLS THIS IS i=%d:::::::::::::\n",i);
		print_stack(stack);
		printf("\ni = %d, infix[i] = '%c', pos = %d, postfix='%s'\n", i, infix[i], pos, postfix);

		if(isNumeric(infix[i]) == 1) { // If the next token is a number
			postfix[pos] = infix[i];
			pos++;
		} else if(infix[i] == '(') {
			push(stack, '(');
		} else if(infix[i] == ')') {
			while(stack->size != 0 && top(stack) != '(') {
				postfix[pos] = top(stack);
				pop(stack);
				pos++;
			}
			pop(stack); // Discard '('
		} else if(isNumeric(infix[i]) == 0) {
			while(stack->size != 0 && prec(top(stack)) > prec(infix[i])) {
				postfix[pos] = top(stack);
				pop(stack);
				pos++;
			}
			push(stack, infix[i]);
		} 
		printf(":::::::::::::::GOODBYE SWEETUMS SEE U NEXT I::::::::::::::::\n");
		
	}

	while(stack->size != 0) {
		postfix[pos] = top(stack);
		pop(stack);
		pos++;
	}
//	postfix[pos+1] = '\0';

	printf("straight to it: '%s'\n",postfix);
/*	int j;
	for(j=0; j<10; j++) {
		printf("%c",postfix[j]);
	} printf("X\n"); */
//	printf("postfix is %s\n",postfix[2]);
	return postfix;
}

/*
 * This function takes in a valid postfix expression
 * and evaluate it to an integer
 *
 * for each token in the string
 *      if the token is numeric
 *          convert it to an integer
 *          push it onto the stack
 *      else
 *          pop the top 2 element off of the stack
 *          the first goes into the right variable
 *          the second goes into the left variable
 *          apply the operation: 
 *              result = left op right
 *              ex result = left + right
 *
 *          push the result onto the stack
 *          
 * return the result from the stack
 */

int evaluate_postfix(char* postfix) {
}

// Testing Main [DEBUG]
int main(int argc, char* argv[]) {
	printf("Testing * before /...\n");
	if(prec('*') > prec('/')) {
		printf("[PASS]\n");
	} else { 
		printf("[FAIL]\n");
	}
	printf("Testing / before %%...\n");
	if(prec('/') > prec('%')) {
		printf("[PASS]\n");
	} else {
		printf("[FAIL]\n");
	}
	printf("Testing %% before +...\n");
	if(prec('%') > prec('+')) {
		printf("[PASS]\n");
	} else {
		printf("[FAIL]\n");
	}
	printf("testing + before -...\n");
	if(prec('+') > prec('-')) {
		printf("[PASS]\n");
	} else {
		printf("[FAIL]\n");
	}
	printf("Testing prec() on non-operator...\n");
	if(prec('x') == 0) {
		printf("[PASS]\n");
	} else {
		printf("[FAIL]\n");
	}

/*	printf("Testing isNumeric with '%s'...\n",argv[1]);
	char tmp = isNumeric(argv[1]);
	if(tmp == 0) {
		printf("[PASS]\n");
	} else {
		printf("[FAIL]\n");
	}
	printf("Testing isNumeric with %s...\n", argv[2]);
	tmp = isNumeric(argv[2]);
	if(tmp == 0) {
		printf("[FAIL]\n");
	} else {
		printf("[PASS]\n");
	}
*/
	

	printf("Trying infix conversion...\n");
//	printf("Infix to postfix on %s...\n",argv[3]);
//	char* str = infix_to_postfix("(5 + 3)\n");
	printf("Result: %s\n", infix_to_postfix("(5 + 3)\n"));
	return 0;
}
